function [G,m,pi,ll,rho,segs] = binomHMMfwdBackViterbiC(ref,depth,chr,posn,cn,mu_0,alphaPrior,betaPrior,pi_0,kappa,txnExpLen,ZS,CNS,maxiter)
%% FUNCTION [G,m,pi,ll,rho,segs] = binomHMMfwdBackViterbiC(ref,depth,chr,posn,cn,mu_0,alphaPrior,betaPrior,pi_0,kappa,txnExpLen,ZS,CNS,maxiter)
%
% author: Gavin Ha <gha@bccrc.ca>
%         Dept of Molecular Oncolgy/Centre for Translational and Applied Genomics
%         British Columbia Cancer Agency
%         University of British Columbia
% date  : March 7, 2011

K = length(mu_0); % prior probability of success
N = length(ref); % number of data points
py = zeros(K,N); % local evidence
mus = zeros(K,maxiter); % state means
pi = pi_0; % initial state distribution   
converged = 0; % flag for convergence
G = zeros(N,1); %latent variable
loglik = zeros(1,maxiter); %log likelihood

%% SET UP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set up the chromosome indicies
% make cell array of chromosome indicies
chrs = unique(chr);
chrsI = cell(1,length(chrs));

piZ = cell(1,length(chrs)); % need 1 initial dist per chromosome
% initialise the chromosome index and the init state distributions 
for c = 1:length(chrs)
    chrsI{c} = find(chr == chrs(c));
    piZ{c} = ones(1,K)/K;
end

%% INITIALISATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
i = 1;
mus(:,1) = mu_0;

% CY matrix; KxN
%disp('binomHMMfwdBackViterbiC: Initializing genotype switch matrix');
%CY = zeros(K,N);
%CY = genotypeSwitchMatrixC(CY,cn,CNS);

% calculate the likelihood conditional on k
for k=1:K
    py(k,:)=binomialpdf(ref,depth,mus(k,i));% + eps;%2.225e-307;    
end
%py = py .* CY;
%py = normalise(py .* CY,1);

% initialise transition matrix to the prior:
disp('binomHMMfwdBackViterbiC: Initializing Non-stationary transition matrix');
A = zeros(K,K);

% Expectation Maximization
loglik(i) = -Inf;
rho = zeros(K,N);    % marginal p(Z_t|Y)
%xi  = zeros(K,K,N);  % marginal p(Z_t,Z_{t-1}|Y); used for estimating A
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
while (~converged && (i < maxiter))
    disp(['binomHMMfwdBackViterbiC: EM iteration:', int2str(i), ' loglik: ',num2str(loglik(i))]);
    i = i+1;
    % E-step
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for c = 1:length(chrsI)
        piZ{c} = pi;
        I = chrsI{c};
        % function [gamma,alpha,beta,loglik] = nonStationaryFwdBack(init_state_distrib,transmat,obslik)
        [rho(:,I),a,b,ll] = fwd_backC_cn(piZ{c}, A, py(:,I),cn(I),CNS,ZS,posn(I),txnExpLen);       
        loglik(i) = loglik(i)+ll;
    end
    %Zcounts = reshape(sum(xi,3),[K,K]);
    
%    disp(Zcounts);
    % M-step
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % update the noise hyperparams
    % function [mu_N, pi] = estimateBinomNoiseParamsMap(x,N,rho,alpha,beta,kappa)
   [mus(:,i), pi] = ...
        estimateBinomNoiseParamsMap(ref',depth',rho,alphaPrior,betaPrior,kappa);

    % re-calculate the likelihood conditional on k
    for k=1:K
        py(k,:)=binomialpdf(ref,depth,mus(k,i));% + eps;%2.225e-307;
    end
    %py = py .* CY;
    %py = normalise(py .* CY,1);
 
    % compute log-likelihood and check convergence
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
    for k=1:K
        priorMu(k) = betapdflog(mus(k,i),alphaPrior(k),betaPrior(k));        
    end
    priorPi = dirichletpdflog(pi,kappa);
    
    disp(['priorMu ',num2Str(sum(priorMu))])
    disp(['priorPi ',num2Str(priorPi)])
    disp(['loglik ',num2Str(loglik(i))])
    loglik(i) = loglik(i) + sum(priorMu) + priorPi;
    if approxeq(loglik(i),loglik(i-1),1)% || (loglik(i) < loglik(i-1))
        converged = 1;
    elseif (loglik(i) < loglik(i-1))
        converged = 1;
        i = i-1;
    end    
end
disp(['Using parameters from EM iteration ',num2str(i)]);
segs = cell(1,length(chrs));
% do the final viterbi pass
for c = 1:length(chrsI)
    piZ{c} = pi;
    I = chrsI{c};
    %function [path loglik] = nonStationaryViterbiPath(prior, transmat,obslik)
    [G(I) ll segs{c}] = viterbi_pathC_cn(log(piZ{c}), A, log(py(:,I)), cn(I), CNS, ZS, posn(I), txnExpLen);
end

m  = mus(:,1:i);
ll = loglik(1:i);
