function [s, pi] = estimateBinomNoiseStromalParamsMap(x,N,rho,rt,rn,alphaS,betaS,kappa)
%% FUNCTION estimateBinomNoiseParamsMap(x,N,rho,alpha,beta,kappa
%
% author: Gavin Ha <gha@bccrc.ca>
%         Sohrab Shah <sshah@cs.ubc.ca>
%         Dept of Molecular Oncolgy/Centre for Translational and Applied Genomics
%         British Columbia Cancer Agency
%         University of British Columbia
% date  : March 7, 2011    
% given the previous estimate of the parameters and the current 
% estimate of the reponsibilities, estimate the new parameters
%
% x      - data, 1xT
% N      - Number of trials, 1xT
% rho    - current estimate of the responsibilites, KxT
% alpha  - parameter to Beta prior, 1xK
% beta   - parameter to Beta prior, 1xK
% kappa  - prior to stationary distribution, 1xK
% RETURN:
% =======
% mu_N     - new MAP estimate of means
% pi       - new MAP estimate of stationary distribution

    
[K,T] = size(rho);
% vectorize all parameters
a = rho * x';
b = rho * (N-x)';
sInterval = [0.01,0.99];
s = fzero(@(s) stromalDerivativeUpdateEqn(s,rt,rn,a,b,alphaS,betaS),sInterval);

% mu_N = (alpha + refCount(I(G_i=g)) - 1)/(alpha + beta + depth - 2) 
%mu_N = (x*rho' + alpha - 1)./(N*rho' + alpha + beta - 2);

% calculate the stationary distribution
% pi(k) = (kappa(k) + sum(rho(k)) - 1)/(sum(kappa) + N - K)
pi = (rho(:,1)+kappa'-1)./(nansum(rho(:,1)) + nansum(kappa) - K);
pi = pi';